# Generated by Django 1.11.23 on 2019-09-18 11:58


import jsonfield.fields

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('cornerstone', '0003_auto_20190621_1000'),
    ]

    operations = [
        migrations.AddField(
            model_name='cornerstoneglobalconfiguration',
            name='languages',
            field=jsonfield.fields.JSONField(default={}, help_text='List of IETF language tags supported by cornerstone'),
        ),
    ]
