# Generated by Django 3.2.19 on 2023-06-08 20:41

from django.db import migrations, models
import enterprise.utils


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0173_auto_20230531_1459'),
    ]

    operations = [
        migrations.AlterField(
            model_name='enterprisecustomerinvitekey',
            name='expiration_date',
            field=models.DateTimeField(default=enterprise.utils.get_default_invite_key_expiration_date, help_text='The key will no longer be valid after this date.'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomerinvitekey',
            name='expiration_date',
            field=models.DateTimeField(default=enterprise.utils.get_default_invite_key_expiration_date, help_text='The key will no longer be valid after this date.'),
        ),
    ]
